"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "SchedulerProcessor", {
    enumerable: true,
    get: ()=>SchedulerProcessor
});
const _aliceBob = require("alice-bob");
const _syncedSet = require("synced-set");
const _schedulerCore = require("./scheduler-core");
const _util = require("./util");
let lastReceivedTime = currentTime;
let SchedulerProcessor = class SchedulerProcessor extends AudioWorkletProcessor {
    blockSize = 128;
    sampleTime = 1 / sampleRate;
    quantumDurationTime = this.blockSize / sampleRate;
    bpm = 120;
    coeff = 1;
    playbackStartTime = 0;
    adjustedStartTime = 0;
    internalTime = 0;
    running = false;
    eventGroups = new _syncedSet.SyncedSet({
        send: (_, cb)=>cb(),
        pick: _schedulerCore.core.pickFromLocal,
        reducer: ()=>null,
        equal: ()=>true
    });
    constructor(){
        super();
        const [worklet] = new _aliceBob.Bob((data)=>this.port.postMessage({
                rpc: data
            }), this).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>{
            if (data.rpc) worklet.receive(data.rpc);
            if (data.eventGroups) this.eventGroups.receive(_schedulerCore.core.deserialize(data.eventGroups));
        };
    }
    async start(playbackStartTime = currentTime) {
        this.running = true;
        this.playbackStartTime = playbackStartTime;
        this.adjustedStartTime = this.playbackStartTime * this.coeff;
        this.internalTime = this.adjustedStartTime;
        lastReceivedTime = playbackStartTime;
        return this.playbackStartTime;
    }
    stop() {
        this.running = false;
    }
    async setBpm(bpm) {
        this.bpm = bpm;
        this.coeff = 1 / (60 * 4 / this.bpm);
        this.sampleTime = 1 / sampleRate / this.coeff;
        this.quantumDurationTime = this.blockSize / sampleRate * this.coeff;
        return this.coeff;
    }
    process() {
        if (!this.running) return true;
        const now = currentTime;
        const elapsedTime = now - lastReceivedTime;
        lastReceivedTime = now;
        this.internalTime += elapsedTime * this.coeff;
        for (const eventGroup of this.eventGroups){
            const events = (0, _util.getEventsInRange)(eventGroup.events, eventGroup.loop, eventGroup.loopStart, eventGroup.loopEnd, this.internalTime, this.sampleTime, this.quantumDurationTime, this.adjustedStartTime);
            for (const target of eventGroup.targets){
                for (const [receivedTime, event] of events){
                    target.midiQueue.push(receivedTime, ...event.midiEvent.data);
                }
            }
        }
        return true;
    }
};
registerProcessor('scheduler', SchedulerProcessor);

//# sourceMappingURL=scheduler-processor.js.map